--[[
TransmissionConfiguration

Specialization for an transmission configuration.

Author:		Ifko[nator]
Date:		02.06.2022

Version:	v1.0
	   
History:	v1.0 @02.06.2022 - initial implementation in FS 22
]]

TransmissionConfiguration = {};
TransmissionConfiguration.currentModName = g_currentModName;

local function getL10nText(text)
	if text:sub(1, 6) == "$l10n_" then
		text = g_i18n:getText(text:sub(7), TransmissionConfiguration.currentModName);
	elseif g_i18n:hasText(text, TransmissionConfiguration.currentModName) then
		text = g_i18n:getText(text, TransmissionConfiguration.currentModName);
	end;

	return text;
end;

function TransmissionConfiguration.prerequisitesPresent(specializations)
    return true;
end;

function TransmissionConfiguration.initSpecialization()
	g_configurationManager:addConfigurationType("transmission", g_i18n:getText("configuration_transmission", TransmissionConfiguration.currentModName), nil, nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION);
	
	local schema = Vehicle.xmlSchema;

	schema:setXMLSpecializationType("TransmissionConfiguration");

	TransmissionConfiguration.registerTransmissionXMLPaths(schema, "vehicle.transmissionConfigurations.transmissionConfiguration(?).extraOption(?)");
end;

function TransmissionConfiguration.registerTransmissionXMLPaths(schema, baseKey)
	schema:register(XMLValueType.FLOAT, baseKey .. ".transmission#minForwardGearRatio", "Min. forward gear ratio");
	schema:register(XMLValueType.FLOAT, baseKey .. ".transmission#maxForwardGearRatio", "Max. forward gear ratio");
	schema:register(XMLValueType.FLOAT, baseKey .. ".transmission#minBackwardGearRatio", "Min. backward gear ratio");
	schema:register(XMLValueType.FLOAT, baseKey .. ".transmission#maxBackwardGearRatio", "Max. backward gear ratio");
	schema:register(XMLValueType.TIME, baseKey .. ".transmission#gearChangeTime", "Gear change time");
	schema:register(XMLValueType.TIME, baseKey .. ".transmission#autoGearChangeTime", "Auto gear change time");
	schema:register(XMLValueType.FLOAT, baseKey .. ".transmission#axleRatio", "Axle ratio", 1);
	schema:register(XMLValueType.FLOAT, baseKey .. ".transmission#startGearThreshold", "Adjusts which gear is used as start gear", VehicleMotor.GEAR_START_THRESHOLD);
	schema:register(XMLValueType.FLOAT, baseKey .. ".transmission.forwardGear(?)#gearRatio", "Gear ratio");
	schema:register(XMLValueType.FLOAT, baseKey .. ".transmission.forwardGear(?)#maxSpeed", "Gear ratio");
	schema:register(XMLValueType.BOOL, baseKey .. ".transmission.forwardGear(?)#defaultGear", "Gear ratio");
	schema:register(XMLValueType.STRING, baseKey .. ".transmission.forwardGear(?)#name", "Gear name to display");
	schema:register(XMLValueType.STRING, baseKey .. ".transmission.forwardGear(?)#reverseName", "Gear name to display (if reverse direction is active)");
	schema:register(XMLValueType.STRING, baseKey .. ".transmission.forwardGear(?)#actionName", "Input Action to select this gear", "SHIFT_GEAR_SELECT_X");
	schema:register(XMLValueType.FLOAT, baseKey .. ".transmission.backwardGear(?)#gearRatio", "Gear ratio");
	schema:register(XMLValueType.FLOAT, baseKey .. ".transmission.backwardGear(?)#maxSpeed", "Gear ratio");
	schema:register(XMLValueType.BOOL, baseKey .. ".transmission.backwardGear(?)#defaultGear", "Gear ratio");
	schema:register(XMLValueType.STRING, baseKey .. ".transmission.backwardGear(?)#name", "Gear name to display");
	schema:register(XMLValueType.STRING, baseKey .. ".transmission.backwardGear(?)#reverseName", "Gear name to display (if reverse direction is active)");
	schema:register(XMLValueType.STRING, baseKey .. ".transmission.backwardGear(?)#actionName", "Input Action to select this gear", "SHIFT_GEAR_SELECT_X");
	schema:register(XMLValueType.STRING, baseKey .. ".transmission.groups#type", "Type of groups (powershift/default)", "default");
	schema:register(XMLValueType.TIME, baseKey .. ".transmission.groups#changeTime", "Change time if default type", 0.5);
	schema:register(XMLValueType.FLOAT, baseKey .. ".transmission.groups.group(?)#ratio", "Ratio while stage active");
	schema:register(XMLValueType.BOOL, baseKey .. ".transmission.groups.group(?)#isDefault", "Is default stage", false);
	schema:register(XMLValueType.STRING, baseKey .. ".transmission.groups.group(?)#name", "Gear name to display");
	schema:register(XMLValueType.STRING, baseKey .. ".transmission.groups.group(?)#actionName", "Input Action to select this group", "SHIFT_GROUP_SELECT_X");
	schema:register(XMLValueType.BOOL, baseKey .. ".transmission.directionChange#useGroup", "Use group as reverse change", false);
	schema:register(XMLValueType.INT, baseKey .. ".transmission.directionChange#reverseGroupIndex", "Group will be activated while direction is changed", 1);
	schema:register(XMLValueType.BOOL, baseKey .. ".transmission.directionChange#useGear", "Use gear as reverse change", false);
	schema:register(XMLValueType.INT, baseKey .. ".transmission.directionChange#reverseGearIndex", "Gear will be activated while direction is changed", 1);
	schema:register(XMLValueType.TIME, baseKey .. ".transmission.directionChange#changeTime", "Direction change time", 0.5);
	schema:register(XMLValueType.BOOL, baseKey .. ".transmission.manualShift#gears", "Defines if gears can be shifted manually", true);
	schema:register(XMLValueType.BOOL, baseKey .. ".transmission.manualShift#groups", "Defines if groups can be shifted manually", true);
	schema:register(XMLValueType.L10N_STRING, baseKey .. ".transmission#name", "Name of transmission to display in the shop");
	schema:register(XMLValueType.STRING, baseKey .. ".transmission#param", "Parameter to insert in transmission name");
end;

function TransmissionConfiguration:loadMotor(superFunc, xmlFile, motorId)
	local key = nil;

	key, motorId = ConfigurationUtil.getXMLConfigurationKey(xmlFile, motorId, "vehicle.motorized.motorConfigurations.motorConfiguration", "vehicle.motorized", "motor");

	local specMotorized = self.spec_motorized;

	local fallbackConfigKey = "vehicle.motorized.motorConfigurations.motorConfiguration(0)";

	specMotorized.motorType = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#type", "vehicle", fallbackConfigKey);
	specMotorized.motorStartAnimation = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#startAnimationName", "vehicle", fallbackConfigKey);
	specMotorized.consumerConfigurationIndex = ConfigurationUtil.getConfigurationValue(xmlFile, key, "#consumerConfigurationIndex", "", 1, fallbackConfigKey);

	local wheelKey, _ = ConfigurationUtil.getXMLConfigurationKey(xmlFile, self.configurations.wheel, "vehicle.wheels.wheelConfigurations.wheelConfiguration", "vehicle.wheels", "wheels");

	ObjectChangeUtil.updateObjectChanges(xmlFile, "vehicle.motorized.motorConfigurations.motorConfiguration", motorId, self.components, self);

	local motorMinRpm = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#minRpm", 1000, fallbackConfigKey);
	local motorMaxRpm = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#maxRpm", 1800, fallbackConfigKey);
	local minSpeed = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#minSpeed", 1, fallbackConfigKey);
	local maxForwardSpeed = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#maxForwardSpeed", nil, fallbackConfigKey);
	local maxBackwardSpeed = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#maxBackwardSpeed", nil, fallbackConfigKey);

	if maxForwardSpeed ~= nil then
		maxForwardSpeed = maxForwardSpeed / 3.6;
	end;

	if maxBackwardSpeed ~= nil then
		maxBackwardSpeed = maxBackwardSpeed / 3.6;
	end;

	local maxWheelSpeed = ConfigurationUtil.getConfigurationValue(xmlFile, wheelKey, ".wheels", "#maxForwardSpeed", nil, nil);

	if maxWheelSpeed ~= nil then
		maxForwardSpeed = maxWheelSpeed / 3.6;
	end;

	local accelerationLimit = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#accelerationLimit", 2, fallbackConfigKey);
	local brakeForce = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#brakeForce", 10, fallbackConfigKey) * 2;
	local lowBrakeForceScale = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#lowBrakeForceScale", 0.5, fallbackConfigKey);
	local lowBrakeForceSpeedLimit = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#lowBrakeForceSpeedLimit", 1, fallbackConfigKey) / 3600;
	local torqueScale = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#torqueScale", 1, fallbackConfigKey);
	local ptoMotorRpmRatio = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#ptoMotorRpmRatio", 4, fallbackConfigKey);
	local transmissionKey = key .. ".transmission";

	if self.configurations.transmission ~= nil then
		local extraOptionNumber = 0;
				
		while true do
			local configurationKey = "vehicle.transmissionConfigurations.transmissionConfiguration(" .. tostring(self.configurations.transmission - 1) .. ")" .. ".extraOption(" .. tostring(extraOptionNumber) .. ")";
				
			if not self.xmlFile:hasProperty(configurationKey) then
				break;
			end;

			local newTransmissionName = Utils.getNoNil(getXMLString(self.xmlFile.handle, configurationKey .. ".transmission#name"), ""); 

			if newTransmissionName ~= "" then
				transmissionKey = configurationKey .. ".transmission";

				break;
			end;

			extraOptionNumber = extraOptionNumber + 1;
		end;
	end;

	if not xmlFile:hasProperty(transmissionKey) then
		transmissionKey = fallbackConfigKey .. ".transmission";
	end;

	local minForwardGearRatio = xmlFile:getValue(transmissionKey .. "#minForwardGearRatio");
	local maxForwardGearRatio = xmlFile:getValue(transmissionKey .. "#maxForwardGearRatio");
	local minBackwardGearRatio = xmlFile:getValue(transmissionKey .. "#minBackwardGearRatio");
	local maxBackwardGearRatio = xmlFile:getValue(transmissionKey .. "#maxBackwardGearRatio");
	local gearChangeTime = xmlFile:getValue(transmissionKey .. "#gearChangeTime");
	local autoGearChangeTime = xmlFile:getValue(transmissionKey .. "#autoGearChangeTime");
	local axleRatio = xmlFile:getValue(transmissionKey .. "#axleRatio", 1);
	local startGearThreshold = xmlFile:getValue(transmissionKey .. "#startGearThreshold", VehicleMotor.GEAR_START_THRESHOLD);

	if maxForwardGearRatio == nil or minForwardGearRatio == nil then
		minForwardGearRatio, maxForwardGearRatio = nil;
	else
		minForwardGearRatio = minForwardGearRatio * axleRatio;
		maxForwardGearRatio = maxForwardGearRatio * axleRatio;
	end;

	if minBackwardGearRatio == nil or maxBackwardGearRatio == nil then
		minBackwardGearRatio, maxBackwardGearRatio = nil;
	else
		minBackwardGearRatio = minBackwardGearRatio * axleRatio;
		maxBackwardGearRatio = maxBackwardGearRatio * axleRatio;
	end;

	local forwardGears = nil;

	if minForwardGearRatio == nil then
		forwardGears = self:loadGears(xmlFile, "forwardGear", transmissionKey, motorMaxRpm, axleRatio, 1);

		if forwardGears == nil then
			print("Warning: Missing forward gear ratios for motor in '" .. self.configFileName .. "'!");

			forwardGears = {{default = false, ratio = 1}};
		end;
	end;

	local backwardGears = nil;

	if minBackwardGearRatio == nil then
		backwardGears = self:loadGears(xmlFile, "backwardGear", transmissionKey, motorMaxRpm, axleRatio, -1);
	end;

	local gearGroups = self:loadGearGroups(xmlFile, transmissionKey .. ".groups", motorMaxRpm, axleRatio);
	local groupsType = xmlFile:getValue(transmissionKey .. ".groups#type", "default");
	local groupChangeTime = xmlFile:getValue(transmissionKey .. ".groups#changeTime", 0.5);
	local directionChangeUseGear = xmlFile:getValue(transmissionKey .. ".directionChange#useGear", false);
	local directionChangeGearIndex = xmlFile:getValue(transmissionKey .. ".directionChange#reverseGearIndex", 1);
	local directionChangeUseGroup = xmlFile:getValue(transmissionKey .. ".directionChange#useGroup", false);
	local directionChangeGroupIndex = xmlFile:getValue(transmissionKey .. ".directionChange#reverseGroupIndex", 1);
	local directionChangeTime = xmlFile:getValue(transmissionKey .. ".directionChange#changeTime", 0.5);
	local manualShiftGears = xmlFile:getValue(transmissionKey .. ".manualShift#gears", true);
	local manualShiftGroups = xmlFile:getValue(transmissionKey .. ".manualShift#groups", true);
	local torqueCurve = AnimCurve.new(linearInterpolator1);
	local torqueI = 0;
	local torqueBase = fallbackConfigKey .. ".motor.torque";

	if key ~= nil and xmlFile:hasProperty(key .. ".motor.torque(0)") then
		torqueBase = key .. ".motor.torque";
	end;

	while true do
		local torqueKey = string.format(torqueBase .. "(%d)", torqueI);
		local normRpm = xmlFile:getValue(torqueKey .. "#normRpm");
		local rpm = nil;

		if normRpm == nil then
			rpm = xmlFile:getValue(torqueKey .. "#rpm");
		else
			rpm = normRpm * motorMaxRpm;
		end;

		local torque = xmlFile:getValue(torqueKey .. "#torque");

		if torque == nil or rpm == nil then
			break;
		end;

		torqueCurve:addKeyframe({
			torque * torqueScale,
			time = rpm
		});

		torqueI = torqueI + 1;
	end;

	specMotorized.motor = VehicleMotor.new(self, motorMinRpm, motorMaxRpm, maxForwardSpeed, maxBackwardSpeed, torqueCurve, brakeForce, forwardGears, backwardGears, minForwardGearRatio, maxForwardGearRatio, minBackwardGearRatio, maxBackwardGearRatio, ptoMotorRpmRatio, minSpeed);

	specMotorized.motor:setGearGroups(gearGroups, groupsType, groupChangeTime);
	specMotorized.motor:setDirectionChange(directionChangeUseGear, directionChangeGearIndex, directionChangeUseGroup, directionChangeGroupIndex, directionChangeTime);
	specMotorized.motor:setManualShift(manualShiftGears, manualShiftGroups);
	specMotorized.motor:setStartGearThreshold(startGearThreshold);

	local rotInertia = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#rotInertia", specMotorized.motor:getRotInertia(), fallbackConfigKey);
	local dampingRateScale = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#dampingRateScale", 1, fallbackConfigKey);

	specMotorized.motor:setRotInertia(rotInertia);
	specMotorized.motor:setDampingRateScale(dampingRateScale);
	specMotorized.motor:setLowBrakeForce(lowBrakeForceScale, lowBrakeForceSpeedLimit);
	specMotorized.motor:setAccelerationLimit(accelerationLimit);

	local motorRotationAccelerationLimit = ConfigurationUtil.getConfigurationValue(xmlFile, key, ".motor", "#rpmSpeedLimit", nil, fallbackConfigKey);

	if motorRotationAccelerationLimit ~= nil then
		motorRotationAccelerationLimit = motorRotationAccelerationLimit * math.pi / 30;

		specMotorized.motor:setMotorRotationAccelerationLimit(motorRotationAccelerationLimit);
	end;

	if gearChangeTime ~= nil then
		specMotorized.motor:setGearChangeTime(gearChangeTime);
	end;

	if autoGearChangeTime ~= nil then
		specMotorized.motor:setAutoGearChangeTime(autoGearChangeTime);
	end;
end

Motorized.loadMotor = Utils.overwrittenFunction(Motorized.loadMotor, TransmissionConfiguration.loadMotor);

function TransmissionConfiguration:processAttributeData(superFunc, storeItem, vehicle, saleItem)
	local dailyUpkeep = 0;
	local powerOutput = 0;
	local transmissionName = nil;
	local fuelCapacity = 0;
	local electricCapacity = 0;
	local methaneCapacity = 0;
	local defCapacity = 0;
	local maxSpeed = 0;
	local capacity = 0;
	local capacityUnit = nil;
	local weight = 0;
	local additionalWeight = 0;
	local workingWidth = 0;
	local workingSpeed = 0;
	local powerNeeded = 0;
	local wheelNames = "";
	local baleSize = "";
	local baleSizeProfile = ShopConfigScreen.GUI_PROFILE.BALE_SIZE_ROUND;
	local wrapperBaleSizeRound, wrapperBaleSizeSquare, loaderBaleSizeRound, loaderBaleSizeSquare, storeItems = nil;

	if storeItem.bundleInfo == nil then
		storeItems = {storeItem};
	else
		storeItems = {};

		for itemNumber = 1, #storeItem.bundleInfo.bundleItems do
			table.insert(storeItems, storeItem.bundleInfo.bundleItems[itemNumber].item);
		end;
	end;

	for itemIndex, item in ipairs(storeItems) do
		StoreItemUtil.loadSpecsFromXML(item);

		local realItem = self.previewVehicles[itemIndex] or vehicle;

		dailyUpkeep = dailyUpkeep + self:processStoreItemUpkeep(item, realItem, saleItem);
		powerOutput = powerOutput + self:processStoreItemPowerOutput(item, realItem, saleItem);
		fuelCapacity = fuelCapacity + self:processStoreItemFuelCapacity(item, realItem, saleItem, FillType.DIESEL);
		electricCapacity = electricCapacity + self:processStoreItemFuelCapacity(item, realItem, saleItem, FillType.ELECTRICCHARGE);
		methaneCapacity = methaneCapacity + self:processStoreItemFuelCapacity(item, realItem, saleItem, FillType.METHANE);
		defCapacity = defCapacity + self:processStoreItemFuelCapacity(item, realItem, saleItem, FillType.DEF);
		transmissionName = transmissionName or Motorized.getSpecValueTransmission(storeItem, realItem, nil, saleItem);

		if vehicle.configurations.transmission ~= nil then
			local extraOptionNumber = 0;
						
			while true do
				local configurationKey = "vehicle.transmissionConfigurations.transmissionConfiguration(" .. tostring(vehicle.configurations.transmission - 1) .. ")" .. ".extraOption(" .. tostring(extraOptionNumber) .. ")";
						
				if not vehicle.xmlFile:hasProperty(configurationKey) then
					break;
				end;
		
				local newTransmissionName = Utils.getNoNil(getXMLString(vehicle.xmlFile.handle, configurationKey .. ".transmission#name"), ""); 
		
				if newTransmissionName ~= "" then
					transmissionName = getL10nText(newTransmissionName);
		
					break;
				end;
		
				extraOptionNumber = extraOptionNumber + 1;
			end;
		end;
		
		local itemCapacity, itemCapacityUnit = self:processStoreItemCapacity(item, realItem, saleItem);

		if capacityUnit ~= nil and itemCapacityUnit ~= nil and itemCapacityUnit ~= capacityUnit then
			print("Warning: Bundled store items have different fill capacity units. Check " .. tostring(storeItem.xmlFilename));
		end;

		capacityUnit = capacityUnit or itemCapacityUnit;
		capacity = capacity + itemCapacity;

		local itemWeight, itemAdditionalWeight = self:processStoreItemWeight(item, realItem, saleItem);

		additionalWeight = additionalWeight + (itemAdditionalWeight or 0);
		weight = weight + (itemWeight or 0);
		workingWidth = math.max(workingWidth, self:processStoreItemWorkingWidth(item, realItem, saleItem));
		workingSpeed = math.max(workingSpeed, self:processStoreItemWorkingSpeed(item, realItem, saleItem));
		maxSpeed = math.max(maxSpeed, self:processStoreItemMaxSpeed(item, realItem, saleItem));
		powerNeeded = powerNeeded + self:processStoreItemPowerNeeded(item, realItem, saleItem);

		local itemBaleSize, itemBaleSizeProfile = self:processStoreItemBalerBaleSize(item);

		if baleSize == "" then
			baleSize = itemBaleSize;
			baleSizeProfile = itemBaleSizeProfile;
		end;

		local roundSize, squareSize = self:processStoreItemBaleWrapperBaleSize(item);

		wrapperBaleSizeRound = wrapperBaleSizeRound or roundSize;
		wrapperBaleSizeSquare = wrapperBaleSizeSquare or squareSize;
		roundSize, squareSize = self:processStoreItemBaleLoaderBaleSize(item);
		loaderBaleSizeRound = loaderBaleSizeRound or roundSize;
		loaderBaleSizeSquare = loaderBaleSizeSquare or squareSize;
	end;

	if vehicle ~= nil then
		for _, name in ipairs(Wheels.getTireNames(vehicle)) do
			if wheelNames ~= "" then
				wheelNames = wheelNames .. " / ";
			end;

			wheelNames = wheelNames .. name;
		end;
	end;

	local values = {};

	if dailyUpkeep ~= 0 then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.MAINTENANCE_COST,
			value = string.format(self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.MAINTENANCE_COST), self.l10n:formatMoney(dailyUpkeep, 2))
		});
	end;

	if powerOutput ~= 0 then
		local hp, kw = self.l10n:getPower(powerOutput)

		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.POWER,
			value = string.format(self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.POWER), MathUtil.round(kw), MathUtil.round(hp))
		});
	end;

	if transmissionName ~= nil then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.TRANSMISSION,
			value = transmissionName
		});
	end;

	if fuelCapacity ~= 0 then
		if defCapacity == 0 then
			table.insert(values, {
				profile = ShopConfigScreen.GUI_PROFILE.FUEL,
				value = string.format(self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.FUEL), fuelCapacity, self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.UNIT_LITER))
			});
		elseif defCapacity > 0 then
			table.insert(values, {
				profile = ShopConfigScreen.GUI_PROFILE.FUEL,
				value = string.format(self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.FUEL_DEF), fuelCapacity, self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.UNIT_LITER), defCapacity, self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.UNIT_LITER), self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.DEF_SHORT))
			});
		end;
	elseif electricCapacity ~= 0 then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.ELECTRICCHARGE,
			value = string.format(self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.FUEL), electricCapacity, self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.UNIT_KW))
		});
	elseif methaneCapacity ~= 0 then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.METHANE,
			value = string.format(self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.FUEL), methaneCapacity, self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.UNIT_KG))
		});
	end;

	if maxSpeed ~= 0 then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.MAX_SPEED,
			value = string.format(self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.MAX_SPEED), string.format("%1d", self.l10n:getSpeed(maxSpeed)), self.l10n:getSpeedMeasuringUnit())
		});
	end;

	if capacity ~= 0 and capacityUnit ~= nil then
		if capacityUnit:sub(1, 6) == "$l10n_" then
			capacityUnit = capacityUnit:sub(7)
		end;

		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.CAPACITY,
			value = string.format(self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.CAPACITY), capacity, self.l10n:getText(capacityUnit))
		});
	end;

	if weight ~= 0 then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.WEIGHT,
			value = g_i18n:formatMass(weight)
		});
	end;

	if additionalWeight ~= 0 then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.ADDITIONAL_WEIGHT,
			value = g_i18n:formatMass(additionalWeight)
		});
	end;

	if powerNeeded ~= 0 then
		local hp, kw = self.l10n:getPower(powerNeeded)

		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.POWER_REQUIREMENT,
			value = string.format(self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.POWER_REQUIREMENT), MathUtil.round(kw), MathUtil.round(hp))
		});
	end;

	if workingWidth ~= 0 then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.WORKING_WIDTH,
			value = string.format(self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.WORKING_WIDTH), g_i18n:formatNumber(workingWidth, 1, true))
		});
	end;

	if workingSpeed ~= 0 then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.WORKING_SPEED,
			value = string.format(self.l10n:getText(ShopConfigScreen.L10N_SYMBOL.WORKING_SPEED), string.format("%1d", self.l10n:getSpeed(workingSpeed)), self.l10n:getSpeedMeasuringUnit())
		});
	end;

	if wheelNames ~= "" then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.WHEELS,
			value = wheelNames
		});
	end;

	if baleSize ~= nil and baleSize ~= "" then
		table.insert(values, {
			profile = baleSizeProfile,
			value = baleSize
		});
	end;

	if wrapperBaleSizeRound ~= nil then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.BALEWRAPPER_SIZE_ROUND,
			value = wrapperBaleSizeRound
		});
	end;

	if wrapperBaleSizeSquare ~= nil then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.BALEWRAPPER_SIZE_SQUARE,
			value = wrapperBaleSizeSquare
		});
	end;

	if loaderBaleSizeRound ~= nil then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.BALE_SIZE_ROUND,
			value = loaderBaleSizeRound
		});
	end;

	if loaderBaleSizeSquare ~= nil then
		table.insert(values, {
			profile = ShopConfigScreen.GUI_PROFILE.BALE_SIZE_SQUARE,
			value = loaderBaleSizeSquare
		});
	end;

	for _ = 1, #self.attributesLayout.elements do
		self.attributesLayout.elements[1]:delete();
	end;

	for _, item in ipairs(values) do
		local itemElement = self.attributeItem:clone(self.attributesLayout);
		local iconElement = itemElement:getDescendantByName("icon");
		local textElement = itemElement:getDescendantByName("text");

		itemElement:reloadFocusHandling(true);
		iconElement:applyProfile(item.profile);
		textElement:setText(item.value);
	end;

	self.attributesLayout:invalidateLayout();
end;

ShopConfigScreen.processAttributeData = Utils.overwrittenFunction(ShopConfigScreen.processAttributeData, TransmissionConfiguration.processAttributeData);